@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2014-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -57,10 +57,13 @@ enum mmpdu_authentication_algorithm_number {
 	MMPDU_AUTH_ALGO_SHARED_KEY,
 	MMPDU_AUTH_ALGO_FT,
 	MMPDU_AUTH_ALGO_SAE,
+	MMPDU_AUTH_ALGO_FILS_SK,
+	MMPDU_AUTH_ALGO_FILS_SK_PFS,
+	MMPDU_AUTH_ALGO_FILS_PK,
 };
 
 /*
- * 802.11-2016, Section 9.4.1.7:
+ * 802.11-2016, Section 9.4.1.7 Reason Code field
  */
 enum mmpdu_reason_code {
 	MMPDU_REASON_CODE_UNSPECIFIED = 1,
@@ -117,44 +120,118 @@ enum mmpdu_reason_code {
 	MMPDU_REASON_CODE_MAC_EXISTS_IN_MBSS = 64,
 	MMPDU_REASON_CODE_MESH_CHAN_REGULATORY = 65,
 	MMPDU_REASON_CODE_MESH_CHAN = 66,
-	MMPDU_REASON_CODE_REJECTED_FOR_SSP_PERMISSION = 67,
-	MMPDU_REASON_CODE_REFUSED_UNAUTHENTICATED_ACCESS = 68,
-	/* 69 - 71 reserved */
-	MMPDU_REASON_CODE_INVALID_RSNE = 72,
-	MMPDU_REASON_CODE_U_APSD_COEXISTANCE_NOTSUPP = 73,
-	MMPDU_REASON_CODE_U_ASPD_COEX_MODE_NOTSUPP = 74,
-	MMPDU_REASON_CODE_BAD_INTERVAL_WITH_U_APSD = 75,
-	MMPDU_REASON_CODE_ANTI_CLOGGING_TOKEN_REQ = 76,
-	MMPDU_REASON_CODE_UNSUPP_FINITE_CYCLIC_GROUP = 77,
-	MMPDU_REASON_CODE_CANNOT_FIND_ALT_TBTT = 78,
-	MMPDU_REASON_CODE_TRANSMISSION_FAILURE = 79,
-	MMPDU_REASON_CODE_REQUESTED_TCLAS_NOTSUPP = 80,
-	MMPDU_REASON_CODE_TCLAS_RESOURCES_EXAUSTED = 81,
-	MMPDU_REASON_CODE_REJECTED_WITH_SUGG_BSS_TRANS = 82,
-	MMPDU_REASON_CODE_REJECT_WITH_SCHEDULE = 83,
-	MMPDU_REASON_CODE_REJECT_NOT_WAKEUP_SPECIFIED = 84,
-	MMPDU_REASON_CODE_SUCCESS_POWER_SAVE_MODE = 85,
-	MMPDU_REASON_CODE_PENDING_ADMITTING_FST_SESSION = 86,
-	MMPDU_REASON_CODE_PERFORMING_FST_NOW = 87,
-	MMPDU_REASON_CODE_PENDING_GAP_IN_BA_WINDOW = 88,
-	MMPDU_REASON_CODE_REJECT_U_PID_SETTING = 89,
-	/* 90 - 91 reserved */
-	MMPDU_REASON_CODE_REFUSED_EXTERNAL_REASON = 92,
-	MMPDU_REASON_CODE_REFUSED_AP_OUT_OF_MEMORY = 93,
-	MMPDU_REASON_CODE_REJECTED_EMERGENCY_SVC_NOTSUPP = 94,
-	MMPDU_REASON_CODE_QUERY_RESPONSE_OUTSTANDING = 95,
-	MMPDU_REASON_CODE_REJECT_DSE_BAND = 96,
-	MMPDU_REASON_CODE_TCLAS_PROCESSING_TERMINATED = 97,
-	MMPDU_REASON_CODE_TS_SCHEDULE_CONFLICT = 98,
-	MMPDU_REASON_CODE_DENIED_WITH_BAND_CHANNEL = 99,
-	MMPDU_REASON_CODE_MCCAOP_RESERVATION_CONFLICT = 100,
-	MMPDU_REASON_CODE_MAF_LIMIT_EXCEEDED = 101,
-	MMPDU_REASON_CODE_MCCA_TRACK_LIMIT_EXCEEDED = 102,
-	MMPDU_REASON_CODE_DENIED_SPECTRUM_MANAGEMENT = 103,
-	MMPDU_REASON_CODE_DENIED_VHT_NOTSUPP = 104,
-	MMPDU_REASON_CODE_ENABLEMENT_DENIED = 105,
-	MMPDU_REASON_CODE_RESTRICTION_FROM_AUTH_GDB = 106,
-	MMPDU_REASON_CODE_AUTHORIZATION_DEENABLED = 107,
+	/* 67 - 65535 reserved */
+};
+
+/*
+ * 802.11-2020, Section 9.4.1.9 Status Code field
+ */
+enum mmpdu_status_code {
+	MMPDU_STATUS_CODE_SUCCESS = 0,
+	MMPDU_STATUS_CODE_UNSPECIFIED = 1,
+	MMPDU_STATUS_CODE_TDLS_REJECTED_ALT_PROV = 2,
+	MMPDU_STATUS_CODE_TDLS_REJECTED = 3,
+	/* 4 reserved */
+	MMPDU_STATUS_CODE_SECURITY_DISABLED = 5,
+	MMPDU_STATUS_CODE_UNACCEPTABLE_LIFETIME = 6,
+	MMPDU_STATUS_CODE_NOT_IN_SAME_BSS = 7,
+	/* 8-9 reserved */
+	MMPDU_STATUS_CODE_CAPABILITIES_MISMATCH = 10,
+	MMPDU_STATUS_CODE_NO_ASSOC_EXISTS = 11,
+	MMPDU_STATUS_CODE_DENIED_OTHER_REASON = 12,
+	MMPDU_STATUS_CODE_UNSUP_AUTH_ALG = 13,
+	MMPDU_STATUS_CODE_TRANS_SEQ_ERROR = 14,
+	MMPDU_STATUS_CODE_CHALLENGE_FAILURE = 15,
+	MMPDU_STATUS_CODE_REJECTED_SEQ_TIMEOUT = 16,
+	MMPDU_STATUS_CODE_DENIED_NO_MORE_STAS = 17,
+	MMPDU_STATUS_CODE_REFUSED_RATE_MISMATCH = 18,
+	MMPDU_STATUS_CODE_NO_SHORT_PREAMBLE_SUPP = 19,
+	/* 20-21 reserved */
+	MMPDU_STATUS_CODE_REJECTED_SPECTRUM_MGMT = 22,
+	MMPDU_STATUS_CODE_REJECTED_BAD_POWER_CAP = 23,
+	MMPDU_STATUS_CODE_REJECTED_BAD_SUPP_CHAN = 24,
+	MMPDU_STATUS_CODE_DENIED_NO_SHORT_SLOT_TIME = 25,
+	/* 26 reserved */
+	MMPDU_STATUS_CODE_DENIED_NO_HT_SUPP = 27,
+	MMPDU_STATUS_CODE_R0KH_UNREACHABLE = 28,
+	MMPDU_STATUS_CODE_DENIED_PCO_TIME_NOT_SUPP = 29,
+	MMPDU_STATUS_CODE_REFUSED_TEMPORARILY = 30,
+	MMPDU_STATUS_CODE_ROBUST_MGMT_POLICY_VIOLATION = 31,
+	MMPDU_STATUS_CODE_UNSPECIFIED_QOS_FAILURE = 32,
+	MMPDU_STATUS_CODE_DENIED_UNSUFFICIENT_BANDWIDTH = 33,
+	MMPDU_STATUS_CODE_DENIED_POOR_CHAN_CONDITIONS = 34,
+	MMPDU_STATUS_CODE_DENIED_QOS_NOT_SUPP = 35,
+	/* 36 reserved */
+	MMPDU_STATUS_CODE_REQUEST_DECLINED = 37,
+	MMPDU_STATUS_CODE_INVALID_PARAMETERS = 38,
+	MMPDU_STATUS_CODE_REJECTED_WITH_SUGG_CHANGES = 39,
+	MMPDU_STATUS_CODE_INVALID_ELEMENT = 40,
+	MMPDU_STATUS_CODE_INVALID_GROUP_CIPHER = 41,
+	MMPDU_STATUS_CODE_INVALID_PAIRWISE_CIPHER = 42,
+	MMPDU_STATUS_CODE_INVALID_AKMP = 43,
+	MMPDU_STATUS_CODE_UNSUPP_RSNE_VERSION = 44,
+	MMPDU_STATUS_CODE_INVALID_RSNE_CAP = 45,
+	MMPDU_STATUS_CODE_CIPHER_OUT_OF_POLICY = 46,
+	MMPDU_STATUS_CODE_REJECTED_FOR_DELAY_PERIOD = 47,
+	MMPDU_STATUS_CODE_DLS_NOT_ALLOWED = 48,
+	MMPDU_STATUS_CODE_NOT_PRESENT = 49,
+	MMPDU_STATUS_CODE_NOT_QOS_STA = 50,
+	MMPDU_STATUS_CODE_LISTEN_INTERVAL_TOO_LARGE = 51,
+	MMPDU_STATUS_CODE_INVALID_FT_ACTION_FRAME_COUNT = 52,
+	MMPDU_STATUS_CODE_INVALID_PMKID = 53,
+	MMPDU_STATUS_CODE_INVALID_MDE = 54,
+	MMPDU_STATUS_CODE_INVALID_FTE = 55,
+	MMPDU_STATUS_CODE_REQ_TCLAS_NOT_SUPP = 56,
+	MMPDU_STATUS_CODE_INSUFFICIENT_TCLAS_PROCESS_RESOURCE = 57,
+	MMPDU_STATUS_CODE_TRY_ANOTHER_BSS = 58,
+	MMPDU_STATUS_CODE_GAS_ADVERTISEMENT_NOT_SUPP = 59,
+	MMPDU_STATUS_CODE_NO_OUTSTANDING_GAS_REQ = 60,
+	MMPDU_STATUS_CODE_GAS_RESP_NOT_RECEIVED = 61,
+	MMPDU_STATUS_CODE_GAS_QUERY_TIMEOUT = 62,
+	MMPDU_STATUS_CODE_GAS_QUERY_RESP_TOO_LARGE = 63,
+	MMPDU_STATUS_CODE_REJECTED_HOME_SUGG_CHANGES = 64,
+	MMPDU_STATUS_CODE_SERVER_UNREACHABLE = 65,
+	/* 66 reserved */
+	MMPDU_STATUS_CODE_REJECTED_FOR_SSP_PERMISSIONS = 67,
+	MMPDU_STATUS_CODE_REFUSED_UNAUTH_ACCESS_NOT_SUPP = 68,
+	/* 69-71 reserved */
+	MMPDU_STATUS_CODE_INVALID_RSNE = 72,
+	MMPDU_STATUS_CODE_U_APSD_COEX_NOT_SUPP = 73,
+	MMPDU_STATUS_CODE_U_APSD_COEX_MODE_NOT_SUPP = 74,
+	MMPDU_STATUS_CODE_BAD_INTERVAL_WITH_U_APSD_COEX = 75,
+	MMPDU_STATUS_CODE_ANTI_CLOGGING_TOKEN_REQ = 76,
+	MMPDU_STATUS_CODE_UNSUPP_FINITE_CYCLIC_GROUP = 77,
+	MMPDU_STATUS_CODE_CANNOT_FIND_ALT_TBTT = 78,
+	MMPDU_STATUS_CODE_TRANSMISSION_FAILURE = 79,
+	/* Spec has 2 errors with the same name (56 above) bug? */
+	MMPDU_STATUS_CODE_REQ_TCLAS_NOT_SUPP2 = 80,
+	MMPDU_STATUS_CODE_TCLAS_RESOURCE_EXAUSTED = 81,
+	MMPDU_STATUS_CODE_REJECTED_WITH_SUGG_BSS_TRANS = 82,
+	MMPDU_STATUS_CODE_REJECT_WITH_SCHEDULE = 83,
+	MMPDU_STATUS_CODE_REJECT_NO_WAKEUP_SPECIFIED = 84,
+	MMPDU_STATUS_CODE_SUCCESS_POWER_SAVE_MODE = 85,
+	MMPDU_STATUS_CODE_PENDING_ADMITTING_FST_SESSION = 86,
+	MMPDU_STATUS_CODE_PERFORMING_FST_NOW = 87,
+	MMPDU_STATUS_CODE_PENDING_GAP_IN_BA_WINDOW = 88,
+	MMPDU_STATUS_CODE_REJECT_U_PID_SETTING = 89,
+	/* 90-91 reserved */
+	MMPDU_STATUS_CODE_REFUSED_EXTERNAL_REASON = 92,
+	MMPDU_STATUS_CODE_REFUSED_AP_OOM = 93,
+	MMPDU_STATUS_CODE_REJECT_EMERGENCY_SERVICE_NOT_SUPP = 94,
+	MMPDU_STATUS_CODE_QUERY_RESPONSE_OUTSTANDING = 95,
+	MMPDU_STATUS_CODE_REJECT_DSE_BAND = 96,
+	MMPDU_STATUS_CODE_TCLAS_PROCESSING_TERM = 97,
+	MMPDU_STATUS_CODE_TS_SCHEDULE_CONFLICT = 98,
+	MMPDU_STATUS_CODE_DENIED_WITH_SUGG_BAND = 99,
+	MMPDU_STATUS_CODE_MCCAOP_RESERVATION_CONFLICT = 100,
+	MMPDU_STATUS_CODE_MAF_LIMIT_EXCEEDED = 101,
+	MMPDU_STATUS_CODE_MCCA_TRACK_LIMIT_EXCEEDED = 102,
+	MMPDU_STATUS_CODE_DENIED_SPECTRUM_MGMT = 103,
+	MMPDU_STATUS_CODE_DENIED_VHT_NOT_SUPP = 104,
+	MMPDU_STATUS_CODE_ENABLEMENT_DENIED = 105,
+	MMPDU_STATUS_CODE_RESTRICT_AUTH_GDB = 106,
+	MMPDU_STATUS_CODE_AUTHORIZATION_DEENABLED = 107,
+	MMPDU_STATUS_CODE_SAE_HASH_TO_ELEMENT = 126,
 };
 
 /* 802.11, Section 8.2.4.1.1, Figure 8-2 */
@@ -333,3 +410,4 @@ struct mmpdu_deauthentication {
 
 const struct mmpdu_header *mpdu_validate(const uint8_t *frame, int len);
 const void *mmpdu_body(const struct mmpdu_header *mpdu);
+size_t mmpdu_header_len(const struct mmpdu_header *mmpdu);
