@@ -67,6 +67,10 @@ struct netconfig {
 	struct resolve *resolve;
 
 	struct l_acd *acd;
+
+	uint32_t addr4_add_cmd_id;
+	uint32_t addr6_add_cmd_id;
+	uint32_t route4_add_gateway_cmd_id;
 };
 
 static struct l_netlink *rtnl;
@@ -585,12 +589,25 @@ static void netconfig_ifaddr_ipv6_cmd_cb(int error, uint16_t type,
 	netconfig_ifaddr_ipv6_notify(type, data, len, user_data);
 }
 
+static void netconfig_route_generic_cb(int error, uint16_t type,
+					const void *data, uint32_t len,
+					void *user_data)
+{
+	if (error) {
+		l_error("netconfig: Failed to add route. Error %d: %s",
+						error, strerror(-error));
+		return;
+	}
+}
+
 static void netconfig_route_add_cmd_cb(int error, uint16_t type,
 						const void *data, uint32_t len,
 						void *user_data)
 {
 	struct netconfig *netconfig = user_data;
 
+	netconfig->route4_add_gateway_cmd_id = 0;
+
 	if (error) {
 		l_error("netconfig: Failed to add route. Error %d: %s",
 						error, strerror(-error));
@@ -608,8 +625,8 @@ static bool netconfig_ipv4_routes_install(struct netconfig *netconfig)
 {
 	L_AUTO_FREE_VAR(char *, gateway) = NULL;
 	struct in_addr in_addr;
-	char *network;
 	char ip[INET_ADDRSTRLEN];
+	char network[INET_ADDRSTRLEN];
 	unsigned int prefix_len =
 		l_rtnl_address_get_prefix_length(netconfig->v4_address);
 
@@ -620,14 +637,13 @@ static bool netconfig_ipv4_routes_install(struct netconfig *netconfig)
 	in_addr.s_addr = in_addr.s_addr &
 				htonl(0xFFFFFFFFLU << (32 - prefix_len));
 
-	network = inet_ntoa(in_addr);
-	if (!network)
+	if (!inet_ntop(AF_INET, &in_addr, network, INET_ADDRSTRLEN))
 		return false;
 
 	if (!l_rtnl_route4_add_connected(rtnl, netconfig->ifindex,
 						prefix_len, network, ip,
 						netconfig->rtm_protocol,
-						netconfig_route_add_cmd_cb,
+						netconfig_route_generic_cb,
 						netconfig, NULL)) {
 		l_error("netconfig: Failed to add subnet route.");
 
@@ -636,18 +652,26 @@ static bool netconfig_ipv4_routes_install(struct netconfig *netconfig)
 
 	gateway = netconfig_ipv4_get_gateway(netconfig);
 	if (!gateway) {
-		l_error("netconfig: Failed to obtain gateway from %s.",
+		l_debug("No gateway obtained from %s.",
 				netconfig->rtm_protocol == RTPROT_STATIC ?
 				"setting file" : "DHCPv4 lease");
 
-		return false;
+		if (netconfig->notify) {
+			netconfig->notify(NETCONFIG_EVENT_CONNECTED,
+						netconfig->user_data);
+			netconfig->notify = NULL;
+		}
+
+		return true;
 	}
 
-	if (!l_rtnl_route4_add_gateway(rtnl, netconfig->ifindex, gateway, ip,
+	netconfig->route4_add_gateway_cmd_id =
+		l_rtnl_route4_add_gateway(rtnl, netconfig->ifindex, gateway, ip,
 						ROUTE_PRIORITY_OFFSET,
 						netconfig->rtm_protocol,
 						netconfig_route_add_cmd_cb,
-						netconfig, NULL)) {
+						netconfig, NULL);
+	if (!netconfig->route4_add_gateway_cmd_id) {
 		l_error("netconfig: Failed to add route for: %s gateway.",
 								gateway);
 
@@ -663,6 +687,8 @@ static void netconfig_ipv4_ifaddr_add_cmd_cb(int error, uint16_t type,
 {
 	struct netconfig *netconfig = user_data;
 
+	netconfig->addr4_add_cmd_id = 0;
+
 	if (error && error != -EEXIST) {
 		l_error("netconfig: Failed to add IP address. "
 				"Error %d: %s", error, strerror(-error));
@@ -685,6 +711,8 @@ static void netconfig_ipv6_ifaddr_add_cmd_cb(int error, uint16_t type,
 	struct netconfig *netconfig = user_data;
 	struct l_rtnl_route *gateway;
 
+	netconfig->addr6_add_cmd_id = 0;
+
 	if (error && error != -EEXIST) {
 		l_error("netconfig: Failed to add IPv6 address. "
 				"Error %d: %s", error, strerror(-error));
@@ -695,7 +723,7 @@ static void netconfig_ipv6_ifaddr_add_cmd_cb(int error, uint16_t type,
 	if (gateway) {
 		L_WARN_ON(!l_rtnl_route_add(rtnl, netconfig->ifindex,
 						gateway,
-						netconfig_route_add_cmd_cb,
+						netconfig_route_generic_cb,
 						netconfig, NULL));
 		l_rtnl_route_free(gateway);
 	}
@@ -748,10 +776,11 @@ static void netconfig_ipv4_dhcp_event_handler(struct l_dhcp_client *client,
 			return;
 		}
 
-		L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
+		L_WARN_ON(!(netconfig->addr4_add_cmd_id =
+				l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
 					netconfig->v4_address,
 					netconfig_ipv4_ifaddr_add_cmd_cb,
-					netconfig, NULL));
+					netconfig, NULL)));
 		break;
 	case L_DHCP_CLIENT_EVENT_LEASE_RENEWED:
 		break;
@@ -807,24 +836,32 @@ static void netconfig_dhcp6_event_handler(struct l_dhcp6_client *client,
 	}
 }
 
+static void netconfig_remove_v4_address(struct netconfig *netconfig)
+{
+	if (!netconfig->v4_address)
+		return;
+
+	L_WARN_ON(!l_rtnl_ifaddr_delete(rtnl, netconfig->ifindex,
+					netconfig->v4_address,
+					netconfig_ifaddr_del_cmd_cb,
+					netconfig, NULL));
+	l_rtnl_address_free(netconfig->v4_address);
+	netconfig->v4_address = NULL;
+}
+
 static void netconfig_reset_v4(struct netconfig *netconfig)
 {
 	if (netconfig->rtm_protocol) {
-		if (netconfig->v4_address) {
-			L_WARN_ON(!l_rtnl_ifaddr_delete(rtnl,
-						netconfig->ifindex,
-						netconfig->v4_address,
-						netconfig_ifaddr_del_cmd_cb,
-						netconfig, NULL));
-			l_rtnl_address_free(netconfig->v4_address);
-			netconfig->v4_address = NULL;
-		}
+		netconfig_remove_v4_address(netconfig);
 
 		l_strfreev(netconfig->dns4_overrides);
 		netconfig->dns4_overrides = NULL;
 
 		l_dhcp_client_stop(netconfig->dhcp_client);
 		netconfig->rtm_protocol = 0;
+
+		l_acd_destroy(netconfig->acd);
+		netconfig->acd = NULL;
 	}
 }
 
@@ -834,10 +871,11 @@ static void netconfig_ipv4_acd_event(enum l_acd_event event, void *user_data)
 
 	switch (event) {
 	case L_ACD_EVENT_AVAILABLE:
-		L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
+		L_WARN_ON(!(netconfig->addr4_add_cmd_id =
+				l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
 					netconfig->v4_address,
 					netconfig_ipv4_ifaddr_add_cmd_cb,
-					netconfig, NULL));
+					netconfig, NULL)));
 		return;
 	case L_ACD_EVENT_CONFLICT:
 		/*
@@ -855,17 +893,18 @@ static void netconfig_ipv4_acd_event(enum l_acd_event event, void *user_data)
 		 * case.
 		 */
 		l_error("netconfig: statically configured address was lost");
-		netconfig_reset_v4(netconfig);
+		netconfig_remove_v4_address(netconfig);
 		break;
 	}
 }
 
 static void netconfig_ipv4_select_and_install(struct netconfig *netconfig)
 {
-	netconfig->v4_address = netconfig_get_static4_address(netconfig);
-	if (netconfig->v4_address) {
-		char ip[INET6_ADDRSTRLEN];
+	char ip[INET6_ADDRSTRLEN];
 
+	netconfig->v4_address = netconfig_get_static4_address(netconfig);
+	if (netconfig->v4_address &&
+			l_rtnl_address_get_address(netconfig->v4_address, ip)) {
 		netconfig->rtm_protocol = RTPROT_STATIC;
 		netconfig->acd = l_acd_new(netconfig->ifindex);
 		l_acd_set_event_handler(netconfig->acd,
@@ -875,17 +914,16 @@ static void netconfig_ipv4_select_and_install(struct netconfig *netconfig)
 			l_acd_set_debug(netconfig->acd, do_debug,
 					"[ACD] ", NULL);
 
-		l_rtnl_address_get_address(netconfig->v4_address, ip);
-
 		if (!l_acd_start(netconfig->acd, ip)) {
 			l_error("failed to start ACD, continuing anyways");
 			l_acd_destroy(netconfig->acd);
 			netconfig->acd = NULL;
 
-			L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
+			L_WARN_ON(!(netconfig->addr4_add_cmd_id =
+				l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
 					netconfig->v4_address,
 					netconfig_ipv4_ifaddr_add_cmd_cb,
-					netconfig, NULL));
+					netconfig, NULL)));
 		}
 
 		return;
@@ -920,9 +958,10 @@ static void netconfig_ipv6_select_and_install(struct netconfig *netconfig)
 	address = netconfig_get_static6_address(netconfig);
 	if (address) {
 		netconfig->rtm_v6_protocol = RTPROT_STATIC;
-		L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex, address,
+		L_WARN_ON(!(netconfig->addr6_add_cmd_id =
+			l_rtnl_ifaddr_add(rtnl, netconfig->ifindex, address,
 					netconfig_ipv6_ifaddr_add_cmd_cb,
-					netconfig, NULL));
+					netconfig, NULL)));
 		l_rtnl_address_free(address);
 		return;
 	}
@@ -965,6 +1004,8 @@ bool netconfig_configure(struct netconfig *netconfig,
 				netconfig_notify_func_t notify, void *user_data)
 {
 	char *mdns;
+	char hostname[HOST_NAME_MAX + 1];
+	bool send_hostname;
 
 	netconfig->dns4_overrides = l_settings_get_string_list(active_settings,
 							"IPv4", "DNS", ' ');
@@ -1005,6 +1046,20 @@ bool netconfig_configure(struct netconfig *netconfig,
 	l_dhcp6_client_set_address(netconfig->dhcp6_client, ARPHRD_ETHER,
 							mac_address, ETH_ALEN);
 
+	if (!l_settings_get_bool(active_settings,
+					"IPv4", "SendHostname", &send_hostname))
+		send_hostname = false;
+
+	if (send_hostname) {
+		if (gethostname(hostname, sizeof(hostname)) == 0) {
+			l_dhcp_client_set_hostname(
+				netconfig->dhcp_client, hostname);
+		} else {
+			l_warn("netconfig: Unable to get hostname. "
+					"Error %d: %s", errno, strerror(errno));
+		}
+	}
+
 	netconfig_ipv4_select_and_install(netconfig);
 
 	netconfig_ipv6_select_and_install(netconfig);
@@ -1034,6 +1089,21 @@ bool netconfig_reset(struct netconfig *netconfig)
 {
 	struct netdev *netdev = netdev_find(netconfig->ifindex);
 
+	if (netconfig->route4_add_gateway_cmd_id) {
+		l_netlink_cancel(rtnl, netconfig->route4_add_gateway_cmd_id);
+		netconfig->route4_add_gateway_cmd_id = 0;
+	}
+
+	if (netconfig->addr4_add_cmd_id) {
+		l_netlink_cancel(rtnl, netconfig->addr4_add_cmd_id);
+		netconfig->addr4_add_cmd_id = 0;
+	}
+
+	if (netconfig->addr6_add_cmd_id) {
+		l_netlink_cancel(rtnl, netconfig->addr6_add_cmd_id);
+		netconfig->addr6_add_cmd_id = 0;
+	}
+
 	if (netconfig->rtm_protocol || netconfig->rtm_v6_protocol)
 		resolve_revert(netconfig->resolve);
 
